function [info,itds,elevs] = h_load_hdr(fname)
% [info,itds,elevs] = h_load_hdr(fname)
%
% Loads HRTF header information from file 'fname.txt'.
% This is functionally equivalent to the older method of loading
% the header information from a MATLAB binary file, as in
% 'load fname.mat'; however, the new method uses ASCII header
% files than can easily be read/written by C programs. See the
% function 'h_new' for a description of the info structure.
%
% The header file has the following format:
%
% % comment
% sampRate numElev numHrtf hrtfFlags numBCoeff numACoeff
% elevs(1,:)
% elevs(2,:)
% itds(1)
% itds(2)
% ...etc
% itds(numHrtf)
%
% Lines beginning with "%" are comments. "sampRate" is the sampling rate,
% "numElev" is the number of elevation angles, "numHrtf" is the total number
% of HRTFs stored (accounting for symmetry), "hrtfFlags" are defined as:
%
% #define HRTF_SYM		0x01
% #define HRTF_ITD		0x02
% #define HRTF_FREQ		0x04
%
% The flags are stored as in decimal notation, not hexadecimal.
% HRTF_SYM means the data is symmetric about the median plane, and thus
% azimuth angles beyond 180 degrees are not stored. When accessing a left
% hemisphere HRTF, the software uses a right hemiphere HRTF and automatically
% swaps the channels. HRTF_ITD means the HRTFs have their ITD extracted and
% stored in the header. HRTF_FREQ means the HRTFs are stored as frequency domain
% magnitudes (using the b coefficients), without phase information. "numBCoeff"
% is the number of "b" coeffs, "numACoeff" is the number of "a" coefficients.
%
% elevs(1,:) is a list of the elevation angles, elevs(2,:) is the corresponding
% number of azimuths at each elevation (without considering symmetry, thus this
% determines the azimuth increment between measurements). The ITDs are stored
% one ITD per line, in the same order as the HRTFs are stored in the datafile.
% ITDs are only written to the header file if the HRTF_ITD flag is set. A positive
% ITD corresponds to a positive azimuth, i.e. a right hemisphere source.
%
% A lengthier description of the Panorama HRTF file format can be found on the
% Wave Arts website, or the document "Wave Arts Panorama HRTF file format.doc".
%
% Copyright (c) 2006 Wave Arts, Inc.
% Written by Bill Gardner
%

path = h_path(fname);
fullname = sprintf('%s.txt',path);
[fp,message] = fopen(fullname,'r');
if fp == -1
	disp(sprintf('cannot open HRTF header %s',fullname));
	error(message);
end

%
% The info line saved in file does not have an entry for the ID
% field. We make space for this after reading.
%
buf = h_getl(fp);
if ~isstr(buf)
	error(sprintf('cannot read info line in HRTF header %s',fullname));
end
[info,count] = sscanf(buf,'%d',[1 6]);
if ~(count == 6)
	error(sprintf('cannot scan info line in HRTF header %s',fullname));
end

%
% Convert info array to internal format, which includes an initial entry
% for HRTF ID. After conversion, we can use HRTF functions that take an hinfo
% structure.
%
info = [0 info];

%
% Read elevs lines
%
n_elev = h_get_nelev(info);
elevs = zeros(2,n_elev);
buf = h_getl(fp);
if ~isstr(buf)
	error(sprintf('cannot read elevs(1,:) line in HRTF header %s',fullname));
end
[elevs(1,:),count] = sscanf(buf,'%d',[1 n_elev]);
if ~(count == n_elev)
	error(sprintf('cannot scan elevs(2,:) line in HRTF header %s',fullname));
end

buf = h_getl(fp);
if ~isstr(buf)
	error(sprintf('cannot read elevs(1,:) line in HRTF header %s',fullname));
end
[elevs(2,:),count] = sscanf(buf,'%d',[1 n_elev]);
if ~(count == n_elev)
	error(sprintf('cannot scan elevs(2,:) line in HRTF header %s',fullname));
end

%
% Read ITDs
%
if (h_is_itdex(info))
	n_itds = h_get_nhrtf(info);
	itds = zeros(n_itds,1);
else
   itds = [];
	n_itds = 0;
end
for i = 1 : n_itds
	buf = h_getl(fp);
	if ~isstr(buf)
		error(sprintf('cannot read itd(%d) in HRTF header %s',i,fullname));
	end
	[itds(i),count] = sscanf(buf,'%f');
	if ~(count == 1)
		error(sprintf('cannot scan itd(%d) in HRTF header %s',i,fullname));
	end
end

fclose(fp);
