function hinfo = h_new(fname,fs,elevs,flags,len,force)
% hinfo = h_new(fname,fs,elevs,flags,len[,force])
%
% Create new HRTF data set.
%
%	fname	filename (without extension). File is created in hrtf
%				directory (see h_path function).
%	fs		sampling rate
%	elevs	elevation matrix, 2 x num_elevs:
%				row 1 contains elevation angles
%				row 2 contains number azimuths at corresponding elev
%	flags	flags:
%				0x01	symmetric data
%				0x02	ITD is extracted from data
%				0x04	stored in frequency domain (num_a_coeffs must be 0)
%	len		length of data [num_b_coeffs num_a_coeffs]
%	force	if true, clobber an existing file
%
% Returns hinfo handle, which is array with following values:
%	id
%	fs
%	num_elevs
%	num_hrtfs
%	flags
%	num_b_coeffs
%	num_a_coeffs
%
% These values should be accessed using the appropriate functions, listed
% below. The h_new function is slow, because an empty data file is written
% to disk. Use h_read and h_write to read and write individual HRTF data.
%
% hinfo access functions:
%	fs = h_get_fs(hinfo)
%	alen = h_get_alen(hinfo)
%	blen = h_get_blen(hinfo)
%	[alen blen] = h_get_len(hinfo)
%	nelev = h_get_nelev(hinfo)
%	elevs = h_get_elevs(hinfo)
%	nhrtf = h_get_nhrtf(hinfo)
%	its = h_get_itds(hinfo)
%	flags = h_get_flags(hinfo)
%	bool = h_is_freq(hinfo)
%	bool = h_is_itdex(hinfo)
%	bool = h_is_sym(hinfo)
%
%	flag = h_flag('sym' or 'itdex' or 'freq');
%
% Copyright (c) 2006 Wave Arts, Inc.
% Written by Bill Gardner
%

use_text_header = 1;	% if 0, use binary MATLAB header

if nargin < 6
	force = 0;
end
global h_cur_id;
if (~exist('h_cur_id') | isempty(h_cur_id))
	h_cur_id = 0;
end
hinfo = zeros(1,7);
hinfo(1) = h_cur_id;
h_cur_id = h_cur_id + 1;

%
% Create full pathname
%
path = h_path(fname);

%
% Declare globals
%
eval(sprintf('global h_fname%d',hinfo(1)));
eval(sprintf('global h_itds%d',hinfo(1)));
eval(sprintf('global h_elevs%d',hinfo(1)));

%
% Set up globals
%
n_hrtfs = h_count_hrtfs(elevs,bit_test(flags,0));
itds = zeros(n_hrtfs,1);
eval(sprintf('h_fname%d = fname;',hinfo(1)));
eval(sprintf('h_itds%d = itds;',hinfo(1)));
eval(sprintf('h_elevs%d = elevs;',hinfo(1)));

%
% Fill hinfo
%
[nr,n_elevs] = size(elevs);
hinfo(2) = fs;
hinfo(3) = n_elevs;
hinfo(4) = n_hrtfs;
hinfo(5) = flags;
hinfo(6) = len(1);
hinfo(7) = len(2);

%
% Now test if data file exists, which is an error unless force is true.
%
if ~force
	fid = fopen(path,'r');
	if ~(fid == -1)
		fclose(fid);
		error(sprintf('%s already exists',path));
	end
end

%
% Create data file
%
disp(sprintf('creating %s',path));
fid = fopen(path,'w','ieee-be');
n_data = n_hrtfs * 2 * (len(1) + len(2));
data = zeros(n_data,1);
fwrite(fid,data,'float32');
fclose(fid);

%
% Save header information
%
if use_text_header
	h_save_hdr(fname,hinfo,itds,elevs);
else
	%
	% Create matlab variable file containing auxiliary information in
	% hinfo, itds, and elevs. When we load this, it will overwrite the
	% local copy of these variables (that's why we use 'info' instead
	% of 'hinfo').
	%
	info = hinfo;
	eval(sprintf('save %s info itds elevs',fname));
end
