function [b, itd, a] = h_read(hinfo, elev, azim)
% [b, itd, a] = h_read(hinfo, elev, azim)
%
% Read HRTF data for a given (elev,azim) location.
% Return stereo B coeffs, ITD in samples, and stereo A coeffs.
% Stereo coeffs are: row 1 = left chl, row 2 = right chl.
%
% If the A coefficient length is 0, read returns [1] for all
% A coefficients.
%
% hinfo is HRTF handle returned by h_new or h_open.
%
% Copyright (c) 2006 Wave Arts, Inc.
% Written by Bill Gardner
%

%
% Check arguments
%
azim = round(azim);
if (azim < 0)
	azim = azim + 360;
elseif (azim >= 360)
	azim = azim - 360;
end
if ((azim < 0) | (azim >= 360))
	error(sprintf('azimuth (%f) must be between -360 and 360 degrees',azim));
end

%
% Get globals
%
elevs = h_get_elevs(hinfo);
itds = h_get_itds(hinfo);
fname = h_get_fname(hinfo);
path = h_path(fname);

%
% Find closest elevation
%
n_elev = h_get_nelev(hinfo);
if (n_elev == 1)
	%
	% Only one elevation in data
	%
	elev_index = 0;
	if ~(elev == elevs(1, 1))
		disp(sprintf('using elev %d for %d', elevs(1,1), elev));
	end
else
	%
	% Search for closest elevation
	%
	for elev_index = 0 : n_elev - 1
		if (elevs(1, elev_index + 1) == elev)
			break;
		elseif (elevs(1, elev_index + 1) > elev)
			if (elev_index > 0) & ...
				(abs(elevs(1, elev_index + 1) - elev) > abs(elevs(1, elev_index) - elev))
				elev_index = elev_index - 1;
			end
%			disp(sprintf('using elev %d for %d', elevs(1, elev_index + 1), elev));
			break;
		end
	end
end

%
% Find closest azimuth. Set flip_flag if symmetric data and left hemisphere.
%
n_azim = elevs(2, elev_index + 1);
azim_index = round(azim * n_azim / 360);

sym_flag = h_is_sym(hinfo);
naz = h_naz(n_azim, sym_flag);
if (sym_flag & (azim_index >= naz))
	flip_flag = 1;
	azim_index = n_azim - azim_index;
%	disp(sprintf('symmetric: azim_index flipped to %d',azim_index));
else
	flip_flag = 0;
end

%
% Open file
%
fid = fopen(path,'r','ieee-be');
if (fid == -1)
	error(sprintf('cannot open %s',path));
end

%
% Seek to data
%
data_index = 0;
for i = 0 : elev_index - 1
	data_index = data_index + h_naz(elevs(2, i + 1), sym_flag);
end
data_index = data_index + azim_index;
[blen, alen] = h_get_len(hinfo);
data_size = 2 * (blen + alen);
byte_index = data_index * data_size * 4;

%disp(sprintf('elev_index %d azim_index %d data_index %d', elev_index, azim_index, data_index)); 
if fseek(fid, byte_index, -1) == -1
	error(sprintf('cannot seek to offset %d',byte_index));
end

%
% Read data
%
[bL,nr] = fread(fid, blen, 'float32');
if ~(nr == blen)
	error('cannot read data');
end

if (alen > 0)
	[aL,nr] = fread(fid, alen, 'float32');
	if ~(nr == alen)
		error('cannot read data');
	end
else
	aL = 1;
end

[bR,nr] = fread(fid, blen, 'float32');
if ~(nr == blen)
	error('cannot read data');
end

if (alen > 0)
	[aR,nr] = fread(fid, alen, 'float32');
	if ~(nr == alen)
		error('cannot read data');
	end
else
	aR = 1;
end

%
% Close file
%
fclose(fid);

%
% Get itd
%
if (h_is_itdex(hinfo))
	itd = itds(data_index + 1);
else
	itd = 0;
end

%
% Flip data if necessary
%
if flip_flag
	tmp = bL;
	bL = bR;
	bR = tmp;
	tmp = aL;
	aL = aR;
   aR = tmp;
   itd = -itd;
end

%
% Return as stereo row vectors.
%
b = [bL' ; bR'];
a = [aL' ; aR'];


